@@ -7,9 +7,8 @@ local rawset, next, table, pairs, print, require, io, os, setmetatable =
 -- defines defaults for unset values. See the
 -- <a href="http://luarocks.org/en/Config_file_format">config
 -- file format documentation</a> for details.
-local x = module("luarocks.cfg")
+module("luarocks.cfg")
 
-local cfg = require("luarocks.cfg")
 local persist = require("luarocks.persist")
 
 local detected = {}
@@ -65,14 +64,14 @@ end
 variables = {}
 rocks_trees = {}
 
-persist.load_into_table(LUAROCKS_SYSCONFIG or sys_config_file, cfg)
+persist.load_into_table(LUAROCKS_SYSCONFIG or sys_config_file, _M)
 
 if not LUAROCKS_FORCE_CONFIG then
    home_config_file = os.getenv("LUAROCKS_CONFIG") or home_config_file
    local home_overrides = persist.load_into_table(home_config_file, { home = home })
    if home_overrides then
       local util = require("luarocks.util")
-      util.deep_merge(cfg, home_overrides)
+      util.deep_merge(_M, home_overrides)
    end
 end
 
@@ -145,7 +144,7 @@ if detected.unix then
    defaults.variables.LUA_BINDIR = LUA_BINDIR or "/usr/local/bin"
    defaults.variables.LUA_INCDIR = LUA_INCDIR or "/usr/local/include"
    defaults.variables.LUA_LIBDIR = LUA_LIBDIR or "/usr/local/lib"
-   defaults.variables.CFLAGS = ""
+   defaults.variables.CFLAGS = "-O2"
    defaults.cmake_generator = "Unix Makefiles"
    defaults.make = "make"
    defaults.platforms = { "unix" }
@@ -165,10 +164,13 @@ if detected.unix then
 end
 
 if detected.cygwin then
-   defaults.lib_extension = "dll"
+   defaults.lib_extension = "so" -- can be overridden in the config file for mingw builds
    defaults.arch = "cygwin-"..proc
    defaults.platforms = {"unix", "cygwin"}
    defaults.cmake_generator = "Unix Makefiles"
+   defaults.variables.CC = "echo -llua | xargs gcc"
+   defaults.variables.LD = "echo -llua | xargs gcc"
+   defaults.variables.LIBFLAG = "-shared"
 end
 
 if detected.macosx then
@@ -204,6 +206,7 @@ end
 defaults.variables.LUA = defaults.lua_interpreter
 defaults.variables.LIB_EXTENSION = defaults.lib_extension
 defaults.variables.OBJ_EXTENSION = defaults.obj_extension
+defaults.variables.LUAROCKS_PREFIX = LUAROCKS_PREFIX
 
 local cfg_mt = {
    __index = function(t, k)
@@ -215,16 +218,16 @@ local cfg_mt = {
    end
 }
 
-if not cfg.variables then
-   cfg.variables = {}
+if not _M.variables then
+   _M.variables = {}
 end
 for k,v in pairs(defaults.variables) do
-   if not cfg.variables[k] then
-      cfg.variables[k] = v
+   if not _M.variables[k] then
+      _M.variables[k] = v
    end
 end
 
-setmetatable(cfg, cfg_mt)
+setmetatable(_M, cfg_mt)
 
 if not next(rocks_trees) then
    if home_tree then
