@@ -37,6 +37,7 @@ local operators = {
 }
 
 local deltas = {
+   scm =    1000,
    cvs =    1000,
    rc =    -1000,
    pre =   -10000,
@@ -169,11 +170,11 @@ end
 local function parse_constraint(input)
    assert(type(input) == "string")
 
-   local op, version, rest = input:match("^([<>=~!]*)%s*([%w%.%_%-]+)[%s,]*(.*)")
+   local no_upgrade, op, version, rest = input:match("^(@?)([<>=~!]*)%s*([%w%.%_%-]+)[%s,]*(.*)")
    op = operators[op]
    version = parse_version(version)
    if not op or not version then return nil end
-   return { op = op, version = version }, rest
+   return { op = op, version = version, no_upgrade = no_upgrade=="@" and true or nil }, rest
 end
 
 --- Convert a list of constraints from string to table format.
@@ -364,7 +365,7 @@ end
 function match_deps(rockspec, blacklist)
    assert(type(rockspec) == "table")
    assert(type(blacklist) == "table" or not blacklist)
-   local matched, missing = {}, {}
+   local matched, missing, no_upgrade = {}, {}, {}
 
    for _, dep in ipairs(rockspec.dependencies) do
       local found = match_dep(dep, blacklist and blacklist[dep.name] or nil)
@@ -373,10 +374,14 @@ function match_deps(rockspec, blacklist)
             matched[dep] = found
          end
       else
-         missing[dep.name] = dep
+         if dep.constraints[1] and dep.constraints[1].no_upgrade then
+            no_upgrade[dep.name] = dep
+         else
+            missing[dep.name] = dep
+         end
       end
    end
-   return matched, missing
+   return matched, missing, no_upgrade
 end
 
 --- Return a set of values of a table.
@@ -407,7 +412,7 @@ function fulfill_dependencies(rockspec)
          local neg, plat = plat:match("^(!?)(.*)")
          if neg == "!" then
             if platforms_set[plat] then
-               return nil, rockspec.package.." is not supported on '"..plat.."' platforms."
+               return nil, "This rockspec for "..rockspec.package.." does not support "..plat.." platforms."
             end
          else
             if platforms_set[plat] then
@@ -420,11 +425,34 @@ function fulfill_dependencies(rockspec)
          end
       end
       if supported == false then
-         return nil, rockspec.package.." is not supported on '".. plat.."' platforms."
+         local plats = table.concat(cfg.platforms, ", ")
+         return nil, "This rockspec for "..rockspec.package.." does not support "..plats.." platforms."
       end
    end
 
-   local matched, missing = match_deps(rockspec)
+   local matched, missing, no_upgrade = match_deps(rockspec)
+
+   if next(no_upgrade) then
+      print("Missing dependencies for "..rockspec.name.." "..rockspec.version..":")
+      for _, dep in pairs(no_upgrade) do
+         print(show_dep(dep))
+      end
+      if next(missing) then
+         for _, dep in pairs(missing) do
+            print(show_dep(dep))
+         end
+      end
+      print()
+      for _, dep in pairs(no_upgrade) do
+         print("This version of "..rockspec.name.." is designed for use with")
+         print(show_dep(dep)..", but is configured to avoid upgrading it")
+         print("automatically. Please upgrade "..dep.name.." with")
+         print("   luarocks install "..dep.name)
+         print("or choose an older version of "..rockspec.name.." with")
+         print("   luarocks search "..rockspec.name)
+      end
+      return nil, "Failed matching dependencies."
+   end
 
    if next(missing) then
       print()
@@ -441,8 +469,9 @@ function fulfill_dependencies(rockspec)
             if not rock then
                return nil, "Could not find a rock to satisfy dependency: "..show_dep(dep)
             end
-            if not install.run(rock) then
-               return nil, "Failed installing dependency: "..rock
+            local ok, err = install.run(rock)
+            if not ok then
+               return nil, "Failed installing dependency: "..rock.." - "..err
             end
          end
       end
