@@ -6,6 +6,7 @@ module("luarocks.fs.win32", package.seeall)
 local cfg = require("luarocks.cfg")
 
 local fs_base_name,
+      fs_copy,
       fs_current_dir,
       fs_execute,
       fs_execute_string,
@@ -15,6 +16,7 @@ local fs_base_name,
 
 function init_fs_functions(impl)
    fs_base_name = impl.base_name
+   fs_copy = impl.copy
    fs_current_dir = impl.current_dir
    fs_execute = impl.execute
    fs_execute_string = impl.execute_string
@@ -59,6 +61,43 @@ function execute_string(cmd)
    end
 end
 
+--- Test for existance of a file.
+-- @param file string: filename to test
+-- @return boolean: true if file exists, false otherwise.
+function exists(file)
+   assert(file)
+   return fs_execute("if not exist " .. fs_Q(file) ..
+                     " invalidcommandname 2>NUL 1>NUL")
+end
+
+--- Test is pathname is a directory.
+-- @param file string: pathname to test
+-- @return boolean: true if it is a directory, false otherwise.
+function is_dir(file)
+   assert(file)
+   return fs_execute("chdir /D " .. fs_Q(file) .. " 2>NUL 1>NUL")
+end
+
+--- Test is file/dir is writable.
+-- @param file string: filename to test
+-- @return boolean: true if file exists, false otherwise.
+function is_writable(file)
+   assert(file)
+   local result
+   if is_dir(file) then
+      local file2 = file .. '/.tmpluarockstestwritable'
+      local fh = io.open(file2, 'w')
+      result = fh ~= nil
+      if fh then fh:close() end
+      os.remove(file2)
+   else
+      local fh = io.open(file, 'r+')
+      result = fh ~= nil
+      if fh then fh:close() end
+   end
+   return result
+end
+
 
 --- Create a directory if it does not already exist.
 -- If any of the higher levels in the path name does not exist
@@ -210,6 +249,16 @@ local function strip_extension(filename)
    return (filename:gsub("%.[^.]+$", "")) or filename
 end
 
+--- Uncompress gzip file.
+-- @param archive string: Filename of archive.
+-- @return boolean : success status
+local function gunzip(archive)
+   local cmd = fs_execute("gunzip -h 1>NUL 2>NUL") and 'gunzip' or
+               fs_execute("gzip   -h 1>NUL 2>NUL") and 'gzip -d'
+   local ok = fs_execute(cmd, archive)
+   return ok
+end
+
 --- Unpack an archive.
 -- Extract the contents of an archive, detecting its format by
 -- filename extension.
@@ -220,12 +269,12 @@ function unpack_archive(archive)
    
    local ok
    if archive:match("%.tar%.gz$") then
-      ok = fs_execute("gunzip ", archive)
+      ok = gunzip(archive)
       if ok then
          ok = fs_execute("tar -xf ", strip_extension(archive))
       end
    elseif archive:match("%.tgz$") then
-      ok = fs_execute("gunzip ", archive)
+      ok = gunzip(archive)
       if ok then
          ok = fs_execute("tar -xf ", strip_extension(archive)..".tar")
       end
@@ -292,23 +341,31 @@ function wrap_script(file, dest)
    return true
 end
 
---- Check if a binaries directory contain platform-specific
--- binaries or only Lua wrappers, and if a libraries directory exists.
--- @param bin_dir string: a binaries directory.
--- @param lib_dir string: a libraries directory.
--- @return boolean: returns true if rock contains platform-specific
--- binary code, or false if it is a pure-Lua rock.
-function is_binary_rock(bin_dir, lib_dir)
-   if fs_exists(lib_dir) then
+function is_actual_binary(name)
+   name = name:lower()
+   if name:match("%.bat$") or name:match("%.exe$") then
       return true
    end
-   if fs_exists(bin_dir) then
-      for _, name in pairs(fs_find(bin_dir)) do
-         name = name:lower()
-         if name:match("%.bat$") or name:match("%.exe$") then
-            return true
-         end
-      end
-   end
    return false
 end
+
+function copy_binary(filename, dest) 
+   local ok, err = fs_copy(filename, dest)
+   if not ok then
+      return nil, err
+   end
+   local exe_pattern = "%.[Ee][Xx][Ee]$"
+   local base = fs_base_name(filename)
+   if base:match(exe_pattern) then
+      base = base:gsub(exe_pattern, ".lua")
+      local helpname = dest.."/"..base
+      local helper = io.open(helpname, "w")
+      if not helper then
+         return nil, "Could not open "..helpname.." for writing."
+      end
+      helper:write('package.path=\"'..package.path:gsub("\\","\\\\")..';\"..package.path\n')
+      helper:write('package.cpath=\"'..package.path:gsub("\\","\\\\")..';\"..package.cpath\n')
+      helper:close()
+   end
+   return true
+end
